module ConfigX
  module Interface
    NAMESPACE_SEPARATOR = '->'.freeze
    PROC_SEPARATOR = '->'.freeze
    TO_H_METHOD_NAME = 'config->to_h'.freeze

    module Base
      def config
        @config ||= NamespaceDelegator.new(self)
      end

      def configure
        yield(config)
      end

      def configure!
        yield(config)
        config_store.freeze
      end

      def clone
        cloned = super
        cloned_store = config_store.clone
        cloned.instance_variable_set(:@config_store, cloned_store)
        cloned.instance_variable_set(:@config, NamespaceDelegator.new(cloned))
        cloned
      end

      define_method TO_H_METHOD_NAME do |map = store_map|
        map.inject({}) do |acc, (k,v)|
          key = k.to_sym
          if v.is_a?(Hash)
            acc[key] = send(TO_H_METHOD_NAME, v)
          else
            acc[key] = self.send(v)
          end

          acc
        end
      end

      private

      def config_store
        @config_store ||= {}
      end

      def evaluate_config_proc_or_value(value, instance_exec: false)
        return unless value
        return value unless value.respond_to?(:call)

        if instance_exec
          value.arity == 1 ? instance_exec(self, &value) : instance_exec(&value)
        else
          value.call
        end
      end

      def store_map
        if is_a?(Class)
          instance_variable_get(:@store_maps)[:class]
        else
          self.class.instance_variable_get(:@store_maps)[:instance]
        end
      end
    end
  end
end
